#!/usr/bin/env python
import sys
import socket
from struct import *
import argparse
import time

class MSocket(object):

    def __init__(self, sock=None):
        if sock is None:
            self.sock = socket.socket(
                socket.AF_INET, socket.SOCK_STREAM)
        else:
            self.sock = sock

    def connect(self, host, port):
        self.sock.connect((host, port))
        self.sock.setblocking(1)

    def send(self, msg):
        totalsent = 0
        while totalsent < len(msg):
            sent = self.sock.send(msg[totalsent:])
            if sent == 0:
                raise RuntimeError("socket connection broken")
            totalsent = totalsent + sent

    def close(self):
        self.sock.shutdown(0)
        self.sock.close()


if __name__ == "__main__":

    SOCKET_CMD_WAIT  = 0xFF05
    SOCKET_CMD_DMAWRITE = 0xFF06
    SOCKET_CMD_REUWRITE = 0xFF07

    parser = argparse.ArgumentParser(description='Sends binary file to the 1541 Ultimate II')
    parser.add_argument('-host', metavar='HOSTNAME', required=True,
                        help='the hostname of the 1541 Ultimate II')
    parser.add_argument('file', metavar='FILE', nargs="?",
                        help='a binary file to send')
    parser.add_argument('-length', metavar='LENGTH',
                        help='max length of file to send')
    parser.add_argument('-address', metavar="ADDRESS",
                        help='start address of the file. Assumes that the file does *not* have a loadaddr')
    parser.add_argument('-delay', metavar='DELAY',
                        help='delay between each transfer')
    parser.add_argument('-reu', action='store_true', help='store to reu')
    parser.add_argument('-sequenceno', action='store_true',
                        help='add a sequence number (byte)')

    args = parser.parse_args()

    # at most 64k + start addr
    readLength = int(('65538', args.length)[args.length != None])

    command = SOCKET_CMD_REUWRITE if args.reu == True else SOCKET_CMD_DMAWRITE

    sequenceno = 0
    while True:
        if args.file != None:
            with open(args.file, "rb") as f:
                data = f.read(readLength) 
        else:
            data = sys.stdin.read(readLength)

        if data == "":
            break

        length = len(data)
        
        #opening and closing a socket for each transfer, had troubles otherwise with lockups for reasons unknown
        sock = MSocket()
        sock.connect(args.host, 64)

        sock.send(pack("<H", command))

        if args.address != None:
            length += 3 if args.reu == True else 2

        if args.sequenceno != None:
            length += 1
            
        sock.send(pack("<H", length))

        if args.address != None:
            if args.reu == True:
                addrbytes = pack("<L", int('0x' + args.address, 16))
                sock.send(addrbytes[0:3])
            else:
                sock.send(pack("<H", int('0x' + args.address, 16)))
        sock.send(data)
        
        if args.sequenceno != None:
            sock.send(pack("<B", sequenceno))
            sequenceno = (sequenceno + 1) % 256

        sock.close()
        
        if args.delay != None:
            sock = MSocket()
            sock.connect(args.host, 64)
            sock.send(pack("<H", SOCKET_CMD_WAIT))
            sock.send(pack("<H", int(args.delay)))
            sock.close()

        if args.file != None:
            break
           
